import os
from multiprocessing import Process

from COMPS.Data import Experiment, Configuration, Priority, Suite

from simtools.ExperimentManager.BaseExperimentManager import BaseExperimentManager
from simtools.ExperimentManager.CompsExperimentManager import CompsExperimentManager
from simtools.SetupParser import SetupParser
from simtools.SimulationCreator.COMPSSimulationCreator import COMPSSimulationCreator
from simtools.SimulationCreator.PythonSimulationCreator import PythonSimulationCreator
from simtools.Utilities.COMPSCache import COMPSCache
from simtools.Utilities.COMPSUtilities import experiment_is_running, COMPS_login, get_semaphore, \
    get_simulation_by_id
from simtools.Utilities.General import init_logging, timestamp

logger = init_logging("COMPSExperimentManager")         # [TODO]: PythonExperimentManager


class PythonExperimentManager(CompsExperimentManager):
    """
    Extends the LocalExperimentManager to manage DTK simulations through COMPSAccess wrappers
    e.g. creation of Simulation, Experiment, Suite objects
    """
    location = 'HPC'

    def __init__(self, experiment=None, config_builder=None):
        # Ensure we use the SetupParser environment of the experiment if it already exists
        super().__init__(experiment, config_builder)

    def get_simulation_creator(self, work_queue):
        return PythonSimulationCreator(config_builder=self.config_builder,
                                      initial_tags=self.exp_builder.tags,
                                      work_queue=work_queue,
                                      experiment=self.experiment,
                                      cache=self.cache,
                                      save_semaphore=self.save_semaphore,
                                      comps_experiment=self.comps_experiment)
