from COMPS import Client
from COMPS.Data.WorkItem import WorkItem, WorkItemFile, WorkItemState
from COMPS.Data.WorkItem import WorkerOrPluginKey

from simtools.AssetManager.FileList import FileList
from simtools.Services.WorkItem.WorkItemSvc import ObjectDataStore
from simtools.Services.Status.StatusSvc import wait_entry_states
from simtools.SetupParser import SetupParser
from WorkOrder import WorkOrder


class WorkItemManager:

    def __init__(self, comps_host=None, comps_env=None, wi_name="DockerWorker WorkItem", wi_type="DockerWorker",
                 docker_image="ubuntu1804python3dtk", plugin_key="1.0.0.0_RELEASE", command=None,
                 user_files=None, wo_kwargs={}, tags={}, check_status=False):
        self.wi = None
        self.wi_name = wi_name
        self.wi_type = wi_type
        self.docker_image = docker_image
        self.plugin_key = plugin_key
        self.comps_host = comps_host
        self.comps_env = comps_env
        self.command = command
        self.wo_kwargs = wo_kwargs
        self.tags = tags
        self.check_status = check_status
        self.user_files = user_files or FileList()
        self._missing_files = []

    def run(self, check_status=False):

        # login
        self.login_comps()

        # Create a WorkItem
        self.wi = WorkItem(self.wi_name, WorkerOrPluginKey("DockerWorker", self.plugin_key), self.comps_env)

        # set tags
        if self.tags:
            self.wi.set_tags(self.tags)

        # Add work order file
        wo = WorkOrder(self.wi_type, self.docker_image, self.command, **self.wo_kwargs)
        self.wi.add_work_order(data=wo.wo_2_dict_str())

        # Add additional files
        for af in self.user_files:
            wi_file = WorkItemFile(af.file_name, "input")
            self.wi.add_file(wi_file, af.absolute_path)

        # Save the work-item to the server
        self.wi.save()
        self.wi.refresh()

        # Refresh local WI db
        self.update_wi_db()

        # Commission
        self.wi.commission()

        if check_status or self.check_status:
            states = [WorkItemState.Succeeded, WorkItemState.Failed, WorkItemState.Canceled]
            wait_entry_states(self.wi, states)
        else:
            print('State -> {} '.format(self.wi.state.name))

    def login_comps(self):
        # update environments
        self.comps_host = self.comps_host or SetupParser.get('server_endpoint')
        self.comps_env = self.comps_env or SetupParser.get('environment')

        # login
        Client.login(self.comps_host)

    def update_wi_db(self):
        ObjectDataStore.create_item(type="WI", provider="COMPS", item_id=str(self.wi.id))

    def add_tag(self, name, value):
        self.tags[name] = value

    def add_file(self, af):
        self.user_files.add_asset_file(af)      # [TODO]: assume user_file is AssetFile?

    def add_wo_arg(self, name, value):
        self.wo_kwargs[name] = value

    def clear_tages(self):
        self.tags = {}

    def clear_user_files(self):
        self.user_files = FileList()

    def clear_wo_args(self):
        self.wo_kwargs = {}
